/* Copyright (C) 2008 Vincent Penquerc'h.
   This file is part of the Ogg/Kate codec library.
   Written by Vincent Penquerc'h.

   This file is preliminary code distributed only as a way to get
   feedback. As such, it is not meant to be finished quality, may
   not work, and streams encoded with it will certainly be unable
   to be decoded by any released version of the Ogg/Kate library.

   Since this code is not supposed to be used by third parties yet,
   there is no licence yet for this file, fair use applies.  */


#ifndef _KATE_CODEC_H_
#define _KATE_CODEC_H_

#include <ogg/ogg.h>

#define KATE_VERSION_MAJOR 0
#define KATE_VERSION_MINOR 0
#define KATE_VERSION_PATCH 0

#define KATE_BITSTREAM_VERSION_MAJOR 0
#define KATE_BITSTREAM_VERSION_MINOR 0

#ifdef __cplusplus
extern "C" {
#endif

#define kate_malloc malloc
#define kate_realloc realloc
#define kate_free free

typedef enum {
  kate_utf8,
} kate_text_encoding;

typedef enum {
  kate_pixel,
  kate_percentage,
} kate_space_metric;

typedef struct kate_region {
  kate_space_metric metric;
  int x;
  int y;
  int w;
  int h;
  size_t style;
} kate_region;

typedef enum {
  kate_hleft,
  kate_hcenter,
  kate_hright,
  kate_hwhole,
} kate_horizontal_alignment;

typedef enum {
  kate_vtop,
  kate_vcenter,
  kate_vbottom,
  kate_vwhole,
} kate_vertical_alignment;

typedef struct kate_color {
  unsigned char r;
  unsigned char g;
  unsigned char b;
  unsigned char a;
} kate_color;

typedef struct kate_style {
  kate_horizontal_alignment halign;
  kate_vertical_alignment valign;
  kate_color text_color;
  kate_color background_color;
} kate_style;

typedef enum {
  kate_curve_none,
  kate_curve_static,
  kate_curve_linear_interpolation,
  kate_curve_catmull_rom_spline,
  kate_curve_bezier_cubic_spline,
} kate_curve_type;

typedef struct kate_curve {
  kate_curve_type type;
  size_t npts;
  double *pts; /* 2D - npts*[2 (x,y) slots] */
} kate_curve;

typedef enum kate_motion_mapping {
  kate_motion_mapping_frame,      /* 0x0 at bottom left of frame, 1x1 at top right of frame */
  kate_motion_mapping_text,       /* 0x0 at bottom left of text, 1x1 at top right of text */
  kate_motion_mapping_region,     /* 0x0 at bottom left of region, 1x1 at top right of region */
  kate_motion_mapping_pixels,     /* motion corresponds to pixel positions, 0x0 at bottom left */
  kate_motion_mapping_user=128    /* 128 to 255 for user specific mappings */
} kate_motion_mapping;

typedef struct kate_motion {
  size_t ncurves;
  kate_curve *curves;
  double *durations;
  kate_motion_mapping mapping;
} kate_motion;

typedef struct kate_info {
  unsigned char bitstream_version_major;
  unsigned char bitstream_version_minor;

  ogg_uint32_t granule_shift;
  ogg_uint32_t gps_numerator; /* granules per second */
  ogg_uint32_t gps_denominator; /* granules per second */

  kate_text_encoding text_encoding;
  char *language;

  char *category;

  size_t nregions;
  kate_region *regions;

  size_t nstyles;
  kate_style *styles;
} kate_info;

struct kate_encode_state; /* internal use */
struct kate_decode_state; /* internal use */

typedef struct kate_state {
  kate_info *ki;
  ogg_int64_t granulepos;
  ogg_int32_t packetno;
  struct kate_encode_state *kes;
  struct kate_decode_state *kds;
} kate_state;

/* this is the same as Vorbis and Theora comments */
typedef struct kate_comment {
  char **user_comments;
  int *comment_lengths;
  int comments;
  char *vendor;
} kate_comment;

/* this is an event passed to the user */
typedef struct kate_event {
  /* the granule at which this event happens */
  ogg_int64_t granulepos;
  /* the granule at which this event ends */
  ogg_int64_t end_granulepos;

  /* description of text */
  kate_text_encoding text_encoding;
  char *language; /* can be NULL, if so defaults to the one in kate_info */
  char *text;
  int len;

  kate_motion motion;   /* may have a zero number of curves if no motion */

  size_t region;
  size_t style;

} kate_event;

extern int kate_get_version(void);
extern const char *kate_get_version_string(void);
extern int kate_get_bitstream_version(void);
extern const char *kate_get_bitstream_version_string(void);

extern int kate_info_init(kate_info *ki);
extern int kate_info_set_language(kate_info *ki,const char *language);
extern int kate_info_add_region(kate_info *ki,const kate_region *kr);
extern int kate_info_add_style(kate_info *ki,const kate_style *ks);
extern int kate_info_matches_language(const kate_info *ki,const char *language);
extern int kate_info_clear(kate_info *ki);

extern int kate_granule_shift(const kate_info *ki);
extern double kate_granule_time(const kate_info *ki,ogg_int64_t granulepos);
extern ogg_int64_t kate_time_granule(const kate_info *ki,double t);
/* extern int kate_granule_smpte(const kate_info *ki,ogg_int64_t granulepos,int *h,int *m,int *s,int *f); */

extern int kate_clear(kate_state *k);

extern int kate_get_motion_point(const kate_motion *km,double t,double *x,double *y);
  
/* comments */
extern int kate_comment_init(kate_comment *kc);
extern int kate_comment_clear(kate_comment *kc);
extern int kate_comment_add(kate_comment *kc,const char *comment);
extern int kate_comment_add_tag(kate_comment *kc,const char *name,const char *value);
extern const char *kate_comment_query(const kate_comment *kc,const char *tag,int count);
extern int kate_comment_query_count(const kate_comment *kc,const char *tag);

/* setup overrides before encoding */
extern int kate_encode_set_language(kate_state *k,const char *language); /* language can be NULL */
extern int kate_encode_set_text_encoding(kate_state *k,kate_text_encoding text_encoding);
extern int kate_encode_set_region(kate_state *k,size_t region);
extern int kate_encode_set_style(kate_state *k,size_t style);

/* encoding */
extern int kate_encode_init(kate_state *k,kate_info *ki);
extern int kate_encode_headers(kate_state *k,kate_comment *kc,ogg_packet *op);
extern int kate_encode_text(kate_state *k,double t0,double t1,const kate_motion *km,const char *text,size_t sz,ogg_packet *op); /* km may be NULL, text is not null terminated */
extern int kate_encode_finish(kate_state *k,ogg_packet *op);

/* decoding */
extern int kate_decode_is_header(const ogg_packet *op);
extern int kate_decode_init(kate_state *k,kate_info *ki);
extern int kate_decode_headerin(kate_info *ki,kate_comment *kc,ogg_packet *op);
extern int kate_decode_packetin(kate_state *k,ogg_packet *op);
extern int kate_decode_eventout(kate_state *k,const kate_event **ev); /* event can be NULL */

/* error codes */
#define KE_INVALID_PARAMETER (-1)    /* a bogus parameter was passed (usually NULL) */
#define KE_OUT_OF_MEMORY (-2)        /* we're running out of cheese, bring some more */
#define KE_BAD_GRANULE (-3)          /* decreasing granule */
#define KE_INIT (-4)                 /* initializing twice, using an uninitialized state, etc */
#define KE_BAD_PACKET (-5)           /* packet is not as expected */

#ifdef __cplusplus
}
#endif

#endif

