/* Copyright (C) 2008 Vincent Penquerc'h.
   This file is part of the Ogg/Kate codec library.
   Written by Vincent Penquerc'h.

   This file is preliminary code distributed only as a way to get
   feedback. As such, it is not meant to be finished quality, may
   not work, and streams encoded with it will certainly be unable
   to be decoded by any released version of the Ogg/Kate library.

   Since this code is not supposed to be used by third parties yet,
   there is no licence yet for this file, fair use applies.  */


#include <stdio.h>
#include <kate/kate.h>

static double catmull_rom(float t,const double *pts)
{
  double t2=t*t,t3=t2*t;

  return 0.5*(
    (2.0*pts[1*2])
    + t * (pts[2*2]-pts[0*2])
    + t2 * (-pts[3*2]+4.0*pts[2*2]-5.0*pts[1*2]+2.0*pts[0*2])
    + t3 * (pts[3*2]-3.0*pts[2*2]+3.0*pts[1*2]-pts[0*2])
  );
}

static double bezier_cubic(float t,const double *pts)
{
  double t2=t*t,t3=t2*t,it=1.0-t,it2=it*it,it3=it2*it;

  return it3*pts[0*2]
         +3.0*t*it2*pts[1*2]
         +3.0*t2*it*pts[2*2]
         +t3*pts[3*2];
}

/* t will be between 0 and 1 */
int kate_get_curve_point(const kate_curve *kc,double t,double *x,double *y)
{
  int nsegs,n;
  float T,t0,t1;

  if (!kc) return KE_INVALID_PARAMETER;
  if (t<0.0 || t>1.0) return KE_INVALID_PARAMETER; /* clamp instead in case of precision problems ? */
  /* x and y may be NULL */

  switch (kc->type) {
    case kate_curve_none:
      /* the motion can be discontinuous */
      return 1;

    case kate_curve_static:
      if (x) *x=kc->pts[0];
      if (y) *y=kc->pts[1];
      return 0;

    case kate_curve_linear_interpolation:
      /* find the segment we're on */
      nsegs=kc->npts-1;
      n=t*nsegs;
      /* ensure it doesn't overflow */
      if (n<0) n=0;
      if (n>=nsegs) n=nsegs-1;
      /* get a 0-1 t on this segment */
      t0=n/(double)nsegs;
      t1=(n+1)/(double)nsegs;
      T=(t-t0)/(t1-t0);
      /* printf("%d/%d pts: segt: %f [%f - %f] -> %f\n",n,kc->npts,t,t0,t1,T); */
      if (x) *x=T*(kc->pts[(n+1)*2+0]-kc->pts[n*2+0]);
      if (y) *y=T*(kc->pts[(n+1)*2+1]-kc->pts[n*2+1]);
      return 0;

    case kate_curve_catmull_rom_spline:
      /* find the segment we're on */
      nsegs=kc->npts-3; /* one segment less than points, and the two end points are dummy */
      n=t*nsegs;
      /* ensure it doesn't overflow */
      if (n<0) n=0;
      if (n>=nsegs) n=nsegs-1;
      /* get a 0-1 t on this segment */
      t0=n/(double)nsegs;
      t1=(n+1)/(double)nsegs;
      T=(t-t0)/(t1-t0);
      /* printf("%d/%d pts: segt: %f [%f - %f] -> %f\n",n,kc->npts,t,t0,t1,T); */
      if (x) *x=catmull_rom(T,kc->pts+2*n);
      if (y) *y=catmull_rom(T,kc->pts+2*n+1);
      return 0;

    case kate_curve_bezier_cubic_spline:
      /* find the segment we're on */
      nsegs=kc->npts-3; /* one segment less than points, and the two end points are dummy */
      n=t*nsegs;
      /* ensure it doesn't overflow */
      if (n<0) n=0;
      if (n>=nsegs) n=nsegs-1;
      /* get a 0-1 t on this segment */
      t0=n/(double)nsegs;
      t1=(n+1)/(double)nsegs;
      T=(t-t0)/(t1-t0);
      /* printf("%d/%d pts: segt: %f [%f - %f] -> %f\n",n,kc->npts,t,t0,t1,T); */
      if (x) *x=bezier_cubic(T,kc->pts+2*n);
      if (y) *y=bezier_cubic(T,kc->pts+2*n+1);
      return 0;

    default:
      return KE_INVALID_PARAMETER;
  }
}

/* t will be between 0 and the sum of all durations in this motion */
int kate_get_motion_point(const kate_motion *km,double t,double *x,double *y)
{
  size_t n;

  if (!km) return KE_INVALID_PARAMETER;
  if (t<0.0) return KE_INVALID_PARAMETER;
  /* x and y may be NULL */

  for (n=0;n<km->ncurves;++n) {
    if (t<=km->durations[n]) {
      return kate_get_curve_point(&km->curves[n],t/km->durations[n],x,y);
    }
    t-=km->durations[n];
  }

  /* t out of range */
  return KE_INVALID_PARAMETER;
}

