/* kate.c
**
** kate dummy packetizing module
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ogg/ogg.h>
#include <kate/kate.h>

#include "oggmerge.h"
#include "kate.h"

typedef struct kate_page_tag {
	oggmerge_page_t *page;
	struct kate_page_tag *next;
} kate_page_t;

typedef struct {
	ogg_sync_state oy;
	int serialno;
	ogg_int64_t old_granulepos;
	int first_page;
        unsigned long int granule_shift;
	unsigned long granule_numerator;
	unsigned long granule_denominator;
	kate_page_t *pages;
} kate_state_t;

static void _add_kate_page(kate_state_t *kstate, ogg_page *og);


int kate_state_init(oggmerge_state_t *state, int serialno)
{
	kate_state_t *kstate;

	if (state == NULL) return 0;

	kstate = (kate_state_t *)malloc(sizeof(kate_state_t));
	if (kstate == NULL) return 0;

	ogg_sync_init(&kstate->oy);
	kstate->first_page = 1;
        kstate->granule_shift=0;
	kstate->granule_numerator = 0;
	kstate->granule_denominator = 0;
	kstate->old_granulepos = 0;

	// NOTE: we just ignore serialno for now
	// until a future libogg supports ogg_page_set_serialno

	state->private = (void *)kstate;

	return 1;
}

int kate_data_in(oggmerge_state_t *state, char *buffer, unsigned long size)
{
	int ret;
	char *buf;
	kate_state_t *kstate;
	ogg_page og;
	ogg_packet op;
	ogg_stream_state os;
	kate_info ki;
	kate_comment kc;

	kstate = (kate_state_t *)state->private;

	// stuff data in
	buf = ogg_sync_buffer(&kstate->oy, size);
	memcpy(buf, buffer, size);
	ogg_sync_wrote(&kstate->oy, size);

	// pull pages out
	while ((ret = ogg_sync_pageout(&kstate->oy, &og)) == 1) {
		if (kstate->first_page) {
			kstate->first_page = 0;
			ogg_stream_init(&os, ogg_page_serialno(&og));
			kate_info_init(&ki);
			kate_comment_init(&kc);
			if (ogg_stream_pagein(&os, &og) < 0) {
				kate_comment_clear(&kc);
				kate_info_clear(&ki);
				ogg_stream_clear(&os);
				return EBADHEADER;
			}
			if (ogg_stream_packetout(&os, &op) != 1) {
				kate_comment_clear(&kc);
				kate_info_clear(&ki);
				ogg_stream_clear(&os);
				return EBADHEADER;
			}
			if (kate_decode_headerin(&ki, &kc, &op) < 0) {
				kate_comment_clear(&kc);
				kate_info_clear(&ki);
				ogg_stream_clear(&os);
				return EBADHEADER;
			}
                        kstate->granule_shift = kate_granule_shift(&ki);
			kstate->granule_numerator = ki.gps_numerator;
			kstate->granule_denominator = ki.gps_denominator;
			kate_comment_clear(&kc);
			kate_info_clear(&ki);
			ogg_stream_clear(&os);
			
		}
		_add_kate_page(kstate, &og);
	}

	if (ret == 0) return EMOREDATA;
	return EOTHER;
}

oggmerge_page_t *kate_page_out(oggmerge_state_t *state)
{
	kate_state_t *kstate;
	oggmerge_page_t *page;

	kstate = (kate_state_t *)state->private;

	if (kstate->pages == NULL) return NULL;

	page = kstate->pages->page;
	kstate->pages = kstate->pages->next;

	return page;
}

static ogg_page *_copy_ogg_page(ogg_page *og)
{
	ogg_page *page;

	page = (ogg_page *)malloc(sizeof(ogg_page));
	if (page == NULL) return NULL;

	page->header_len = og->header_len;
	page->body_len = og->body_len;
	page->header = (unsigned char *)malloc(page->header_len);
	if (page->header == NULL) {
		free(page);
		return NULL;
	}
	memcpy(page->header, og->header, page->header_len);
	page->body = (unsigned char *)malloc(page->body_len);
	if (page->body == NULL) {
		free(page->header);
		free(page);
		return NULL;
	}
	memcpy(page->body, og->body, page->body_len);

	return page;
}

static u_int64_t _make_timestamp(kate_state_t *kstate, ogg_int64_t granulepos)
{
	u_int64_t stamp;
	
	if (kstate->granule_denominator == 0) return 0;
	
        /* Kate is start time timed, so we use the new granule, not the old one */
        stamp = (granulepos>>kstate->granule_shift)*1000000.0*kstate->granule_denominator/(double)kstate->granule_numerator;

	kstate->old_granulepos = granulepos;

	return stamp;
}

static void _add_kate_page(kate_state_t *kstate, ogg_page *og)
{
	oggmerge_page_t *page;
	kate_page_t *kpage, *temp;

	// build oggmerge page
	page = (oggmerge_page_t *)malloc(sizeof(oggmerge_page_t));
	page->og = _copy_ogg_page(og);
	page->timestamp = _make_timestamp(kstate, ogg_page_granulepos(og));
	
	// build kate page
	kpage = (kate_page_t *)malloc(sizeof(kate_page_t));
	if (kpage == NULL) {
		free(page->og->header);
		free(page->og->body);
		free(page->og);
		free(page);
		return;
	}

	kpage->page = page;
	kpage->next = NULL;

	// add page to state
	temp = kstate->pages;
	if (temp == NULL) {
		kstate->pages = kpage;
	} else {
		while (temp->next) temp = temp->next;
		temp->next = kpage;
	}
}














